//************************************************
//
//	VStatusBar.cpp
//	A status bar class with no text labels.
//  Vikram Kulkarni
//	Copyright 1997
//
//************************************************
//
//  Very similar to BStatusBar...
//  BStatusBar:
//		Copyright 1996-97, Be Incorporated
//
//************************************************

#include "VStatusBar.h"

VStatusBar::VStatusBar ( BRect frame,
						  const char *name,
						  uint32 resizingMode,
						  uint32 flags )
	: BView ( frame, name, resizingMode, flags )
{
	fMax = 100.0;
	fCurrent = 0.0;
	fBarHeight = 13.0;

	fBarColor.red = 50;
	fBarColor.green = 150;
	fBarColor.blue = 255;

	fLightBarColor.red = 132;
	fLightBarColor.green = 192;
	fLightBarColor.blue = 255;

	fDarkBarColor.red = 30;
	fDarkBarColor.green = 90;
	fDarkBarColor.blue = 153;
	
	fDarkGrey.red = fDarkGrey.green = fDarkGrey.blue = 86;
	fLightGrey.red = fLightGrey.green = fLightGrey.blue = 172;
	fMedGrey.red = fMedGrey.green = fMedGrey.blue = 129;
	
	SetHighColor(fBarColor);
}

VStatusBar::VStatusBar(BMessage *data)
	: BView(data)
{
	fMax = data->FindFloat("fMax");
	fCurrent = data->FindFloat("fCurrent");
	fBarHeight = data->FindFloat("fBarHeight");
	rgb_color *color;
	ssize_t size = sizeof(rgb_color);
	data->FindData("fBarColor", B_RGB_COLOR_TYPE, (const void **) &color, &size);
	SetBarColor(*color);

	fDarkGrey.red = fDarkGrey.green = fDarkGrey.blue = 86;
	fLightGrey.red = fLightGrey.green = fLightGrey.blue = 172;
	fMedGrey.red = fMedGrey.green = fMedGrey.blue = 129;

}

VStatusBar::~VStatusBar()
{
}

VStatusBar * VStatusBar::Instantiate(BMessage *data)
{
	if ( validate_instantiation(data, "VStatusBar") )
		return new VStatusBar(data);
	
	return NULL; 
}

status_t VStatusBar::Archive(BMessage *data, bool deep) const
{
	BView::Archive(data, deep);
	data->AddString("class", "VStatusBar");
	data->AddFloat("fMax", fMax);
	data->AddFloat("fCurrent", fCurrent);
	data->AddFloat("fBarHeight", fBarHeight);
	data->AddData("fBarColor", B_RGB_COLOR_TYPE, &fBarColor, sizeof(fBarColor));
	return B_OK;
}

void VStatusBar::AttachedToWindow()
{
	ResizeTo(Bounds().Width(), fBarHeight + 4);
}

void VStatusBar::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		B_UPDATE_STATUS_BAR:
		{
			fCurrent += msg->FindFloat("delta");
			Invalidate();
			break;
		}
		
		B_RESET_STATUS_BAR:
		{
			fCurrent = 0.0;
			Invalidate();
			break;
		}
		
		default:
		{
			BView::MessageReceived(msg);
			break;
		}
	}
}

void VStatusBar::Draw ( BRect updateRect )
{
	float width = Bounds().Width();
	float height = Bounds().Height();
	float progress_width = (fCurrent/fMax) * (width - 6);
	if (progress_width > (width - 6))
		progress_width = width - 6;
	BView::Draw ( updateRect );
		
	BeginLineArray(10);
	
	AddLine(BPoint(0,0), BPoint(width, 0), fLightGrey);
	AddLine(BPoint(0,0), BPoint(0, height), fLightGrey);
	
	AddLine(BPoint(1,1), BPoint(width -  1, 1), fDarkGrey);
	AddLine(BPoint(1,1), BPoint(1, height -  1), fDarkGrey);
	
	AddLine(BPoint(width - 1, 2), BPoint(width - 1, height - 1), fMedGrey);
	AddLine(BPoint(2, height - 1), BPoint(width - 1, height - 1), fMedGrey);
	
	AddLine(BPoint(2,2), BPoint(3 + progress_width, 2), fLightBarColor);
	AddLine(BPoint(2,2), BPoint(2, height - 3), fLightBarColor);
	
	AddLine(BPoint(4 + progress_width, 2), BPoint(4 + progress_width, height - 3), fDarkBarColor);
	AddLine(BPoint(2, height - 2), BPoint(4 + progress_width, height - 2), fDarkBarColor);
	
	EndLineArray();
	
	FillRect(BRect( BPoint(3,3), BPoint(3 + progress_width, height - 3)));	
}

void VStatusBar::SetBarColor ( rgb_color color)
{
	fBarColor.red = color.red;
	fBarColor.green = color.green;
	fBarColor.blue = color.blue;
	fBarColor.alpha = color.alpha;

	// fBarColor + 40% of what's left to go
	// ie fBarColor + 40% of (255 - fBarColor)
	fLightBarColor.red = fBarColor.red + (2*(255 - fBarColor.red) / 5 );
	fLightBarColor.green = fBarColor.green + (2*(255 - fBarColor.green) / 5 );
	fLightBarColor.blue = fBarColor.blue + (2*(255 - fBarColor.blue) / 5 );
	fLightBarColor.alpha = fBarColor.alpha;

	// 60% of fBarColor
	fDarkBarColor.red =  3 * fBarColor.red / 5;
	fDarkBarColor.green = 3 * fBarColor.green / 5;
	fDarkBarColor.blue = 3 * fBarColor.blue / 5;
	fDarkBarColor.alpha = fBarColor.alpha;

	SetHighColor(fBarColor);
}

void VStatusBar::SetBarHeight ( float height )
{
	fBarHeight = height;
	ResizeTo( Bounds().Width(), fBarHeight + 4 );
}

void VStatusBar::SetMaxValue ( float max )
{
	fMax = max;
}

void VStatusBar::Update ( float delta )
{
	if (fCurrent != fMax)
	{
		fCurrent += delta;
		Invalidate();
	}
}

void VStatusBar::Reset ()
{
	fCurrent = 0.0;
	Invalidate();
}

float VStatusBar::CurrentValue () const
{
	return (fCurrent);
}

float VStatusBar::MaxValue () const
{
	return (fMax);
}

rgb_color VStatusBar::BarColor () const
{
	return (fBarColor);
}

float VStatusBar::BarHeight() const
{
	return (fBarHeight);
}

